@@ -4,23 +4,29 @@ package ORLite;
 
 use 5.006;
 use strict;
-use Carp         ();
-use File::Spec   ();
-use File::Temp   ();
-use Params::Util qw{ _STRING _CLASS _HASH };
-use DBI          ();
-# use DBD::SQLite ();
-BEGIN {
-	# DBD::SQLite has a bug that generates a spurious warning
-	# at compile time, so we need to temporarily disable them.
-	# Remove this hack once DBD::SQLite fixes the bug.
-	local $^W = 0;
-	require DBD::SQLite;
-}
+use Carp              ();
+use File::Spec   0.80 ();
+use File::Temp   0.20 ();
+use File::Path   2.04 ();
+use File::Basename  0 ();
+use Params::Util 0.33 ();
+use DBI         1.607 ();
+use DBD::SQLite  1.25 ();
 
 use vars qw{$VERSION};
 BEGIN {
-	$VERSION = '0.15';
+	$VERSION = '1.26_01';
+	$VERSION = eval $VERSION;
+}
+
+# Support for the 'prune' option
+my @PRUNE = ();
+END {
+	foreach ( @PRUNE ) {
+		next unless -e $_;
+		require File::Remove;
+		File::Remove::remove($_);
+	}
 }
 
 
@@ -35,14 +41,14 @@ sub import {
 
 	# Check for debug mode
 	my $DEBUG = 0;
-	if ( defined _STRING($_[-1]) and $_[-1] eq '-DEBUG' ) {
+	if ( defined Params::Util::_STRING($_[-1]) and $_[-1] eq '-DEBUG' ) {
 		$DEBUG = 1;
 		pop @_;
 	}
 
 	# Check params and apply defaults
 	my %params;
-	if ( defined _STRING($_[1]) ) {
+	if ( defined Params::Util::_STRING($_[1]) ) {
 		# Support the short form "use ORLite 'db.sqlite'"
 		%params = (
 			file     => $_[1],
@@ -50,7 +56,7 @@ sub import {
 			package  => undef, # Automatic
 			tables   => 1,
 		);
-	} elsif ( _HASH($_[1]) ) {
+	} elsif ( Params::Util::_HASHLIKE($_[1]) ) {
 		%params = %{ $_[1] };
 	} else {
 		Carp::croak("Missing, empty or invalid params HASH");
@@ -59,7 +65,7 @@ sub import {
 		$params{create} = 0;
 	}
 	unless (
-		defined _STRING($params{file})
+		defined Params::Util::_STRING($params{file})
 		and (
 			$params{create}
 			or
@@ -77,17 +83,32 @@ sub import {
 	unless ( defined $params{package} ) {
 		$params{package} = scalar caller;
 	}
-	unless ( _CLASS($params{package}) ) {
+	unless ( Params::Util::_CLASS($params{package}) ) {
 		Carp::croak("Missing or invalid package class");
 	}
 
 	# Connect to the database
 	my $file     = File::Spec->rel2abs($params{file});
+	my $created  = ! -f $params{file};
+	if ( $created ) {
+		# Create the parent directory
+		my $dir = File::Basename::dirname($file);
+		unless ( -d $dir ) {
+			my @dirs = File::Path::mkpath( $dir, { verbose => 0 } );
+			push @PRUNE, @dirs if $params{prune};
+		}
+		push @PRUNE, $file if $params{prune};
+	}
 	my $pkg      = $params{package};
 	my $readonly = $params{readonly};
 	my $dsn      = "dbi:SQLite:$file";
 	my $dbh      = DBI->connect($dsn);
 
+	# Schema creation support
+	if ( $created and Params::Util::_CODELIKE($params{create}) ) {
+		$params{create}->( $dbh );
+	}
+
 	# Check the schema version before generating
 	my $version  = $dbh->selectrow_arrayref('pragma user_version')->[0];
 	if ( exists $params{user_version} and $version != $params{user_version} ) {
@@ -95,13 +116,19 @@ sub import {
 	}
 
 	# Generate the support package code
-	my $code  = <<"END_PERL";
+	my $code = <<"END_PERL";
 package $pkg;
 
 use strict;
+use Carp ();
+use DBI  ();
 
 my \$DBH = undef;
 
+sub orlite { '$VERSION' }
+
+sub sqlite { '$file' }
+
 sub dsn { '$dsn' }
 
 sub dbh {
@@ -114,6 +141,10 @@ sub connect {
 	DBI->connect(\$_[0]->dsn);
 }
 
+sub prepare {
+	shift->dbh->prepare(\@_);
+}
+
 sub do {
 	shift->dbh->do(\@_);
 }
@@ -142,15 +173,22 @@ sub selectrow_hashref {
 	shift->dbh->selectrow_hashref(\@_);
 }
 
-sub prepare {
-	shift->dbh->prepare(\@_);
-}
-
 sub pragma {
 	\$_[0]->do("pragma \$_[1] = \$_[2]") if \@_ > 2;
 	\$_[0]->selectrow_arrayref("pragma \$_[1]")->[0];
 }
 
+sub iterate {
+	my \$class = shift;
+	my \$call  = pop;
+	my \$sth   = \$class->prepare( shift );
+	\$sth->execute( \@_ );
+	while ( \$_ = \$sth->fetchrow_arrayref ) {
+		\$call->() or last;
+	}
+	\$sth->finish;
+}
+
 END_PERL
 
 	# Add transaction support if not readonly
@@ -170,6 +208,16 @@ sub commit {
 	return 1;
 }
 
+sub commit_begin {
+	if ( \$DBH ) {
+		\$DBH->commit;
+		\$DBH->begin_work;
+	} else {
+		\$_[0]->begin;
+	}
+	return 1;
+}
+
 sub rollback {
 	\$DBH or return 1;
 	\$DBH->rollback;
@@ -178,14 +226,24 @@ sub rollback {
 	return 1;
 }
 
+sub rollback_begin {
+	if ( \$DBH ) {
+		\$DBH->rollback;
+		\$DBH->begin_work;
+	} else {
+		\$_[0]->begin;
+	}
+	return 1;
+}
+
 END_PERL
 
 	# Optionally generate the table classes
 	if ( $params{tables} ) {
 		# Capture the raw schema information
 		my $tables = $dbh->selectall_arrayref(
-			'select * from sqlite_master where type = ?',
-			{ Slice => {} }, 'table',
+			'select * from sqlite_master where name not like ? and type = ?',
+			{ Slice => {} }, 'sqlite_%', 'table',
 		);
 		foreach my $table ( @$tables ) {
 			$table->{columns} = $dbh->selectall_arrayref(
@@ -202,8 +260,7 @@ END_PERL
 			$table->{cindex} = map { $_->{name} => $_ } @columns;
 
 			# Discover the primary key
-			$table->{pk}     = List::Util::first { $_->{pk} } @columns;
-			$table->{pk}     = $table->{pk}->{name} if $table->{pk};
+			@{$table->{pk}}  = map($_->{name}, grep { $_->{pk} } @columns);
 
 			# What will be the class for this table
 			$table->{class}  = ucfirst lc $table->{name};
@@ -243,14 +300,18 @@ END_PERL
 		# Generate the per-table code
 		foreach my $table ( @$tables ) {
 			# Generate the accessors
-			my $sql       = $table->{sql};
-			my @columns   = @{ $table->{columns} };
-			my @names     = map { $_->{name} } @columns;
+			my $sql     = $table->{sql};
+			my @columns = @{ $table->{columns} };
+			my @names   = map { $_->{name} } @columns;
 
 			# Generate the elements in all packages
 			$code .= <<"END_PERL";
 package $table->{class};
 
+sub base { '$pkg' }
+
+sub table { '$table->{name}' }
+
 sub select {
 	my \$class = shift;
 	my \$sql   = '$sql->{select} ';
@@ -267,12 +328,31 @@ sub count {
 	$pkg->selectrow_array( \$sql, {}, \@_ );
 }
 
+sub iterate {
+	my \$class = shift;
+	my \$call  = pop;
+	my \$sql   = '$sql->{select} ';
+	   \$sql  .= shift if \@_;
+	my \$sth   = $pkg->prepare( \$sql );
+	\$sth->execute( \@_ );
+	while ( \$_ = \$sth->fetchrow_hashref ) {
+		bless( \$_, '$table->{class}' );
+		\$call->() or last;
+	}
+	\$sth->finish;
+}
+
 END_PERL
 
 			# Generate the elements for tables with primary keys
 			if ( defined $table->{pk} and ! $readonly ) {
 				my $nattr = join "\n", map { "\t\t$_ => \$attr{$_}," } @names;
 				my $iattr = join "\n", map { "\t\t\$self->{$_},"       } @names;
+				my $fill_pk = scalar @{$table->{pk}} == 1 
+					    ? "\t\$self->{$table->{pk}->[0]} = \$dbh->func('last_insert_rowid') unless \$self->{$table->{pk}->[0]};"
+					    : q{};
+				my $where_pk      = join(' and ', map("$_ = ?", @{$table->{pk}}));
+				my $where_pk_attr = join("\n", map("\t\t\$self->{$_},", @{$table->{pk}}));				
 				$code .= <<"END_PERL";
 
 sub new {
@@ -293,15 +373,16 @@ sub insert {
 	\$dbh->do('$sql->{insert}', {},
 $iattr
 	);
-	\$self->{$table->{pk}} = \$dbh->func('last_insert_rowid') unless \$self->{$table->{pk}};
+$fill_pk	
 	return \$self;
 }
 
 sub delete {
 	my \$self = shift;
 	return $pkg->do(
-		'delete from $table->{name} where $table->{pk} = ?',
-		{}, \$self->{$table->{pk}},
+		'delete from $table->{name} where $where_pk',
+		{}, 
+$where_pk_attr		
 	) if ref \$self;
 	Carp::croak("Must use truncate to delete all rows") unless \@_;
 	return $pkg->do(
@@ -310,6 +391,10 @@ sub delete {
 	);
 }
 
+sub truncate {
+	$pkg->do( 'delete from $table->{name}', {} );
+}
+
 END_PERL
 
 			}
@@ -317,7 +402,7 @@ END_PERL
 		# Generate the accessors
 		$code .= join "\n\n", map { $_->{fk} ? <<"END_DIRECT" : <<"END_ACCESSOR" } @columns;
 sub $_->{name} {
-	($_->{fk}->[1]->{class}\->select('where $_->{fk}->[1]->{pk} = ?', \$_[0]->{$_->{name}}))[0];
+	($_->{fk}->[1]->{class}\->select('where $_->{fk}->[1]->{pk}->[0] = ?', \$_[0]->{$_->{name}}))[0];
 }
 END_DIRECT
 sub $_->{name} {
@@ -329,25 +414,15 @@ END_ACCESSOR
 	}
 	$dbh->disconnect;
 
+	# Add any custom code to the end
+	if ( defined $params{append} ) {
+		$code .= "\npackage $pkg;\n" if $params{tables};
+		$code .= "\n$params{append}";
+	}
+
 	# Load the code
 	if ( $DEBUG ) {
-		# Compile the combined code via a temp file
-		my ($fh, $filename) = File::Temp::tempfile();
-		$fh->print("$code\n\n1;\n");
-		close $fh;
-		require $filename;
-		unlink $filename;
-
-		# Print the debugging output
-		my @trace = map {
-			s/\s*[{;]$//;
-			s/^s/  s/;
-			s/^p/\np/;
-			"$_\n"
-		} grep {
-			/^(?:package|sub)\b/
-		} split /\n/, $code;
-		print STDERR @trace, "\n$pkg code saved as $filename\n\n";
+		dval("$code\n\n1;\n");
 	} else {
 		eval("$code\n\n1;\n");
 		die $@ if $@;
@@ -356,6 +431,28 @@ END_ACCESSOR
 	return 1;
 }
 
+sub dval {
+	# Write the code to the temp file
+	my ($fh, $filename) = File::Temp::tempfile();
+	$fh->print($_[0]);
+	close $fh;
+	require $filename;
+	unlink $filename;
+
+	# Print the debugging output
+	my @trace = map {
+		s/\s*[{;]$//;
+		s/^s/  s/;
+		s/^p/\np/;
+		"$_\n"
+	} grep {
+		/^(?:package|sub)\b/
+	} split /\n/, $_[0];
+	# print STDERR @trace, "\nCode saved as $filename\n\n";
+
+	return 1;
+}
+
 1;
 
 __END__
@@ -370,6 +467,8 @@ ORLite - Extremely light weight SQLite-specific ORM
 
   package Foo;
   
+  # Simplest possible usage
+  
   use strict;
   use ORLite 'data/sqlite.db';
   
@@ -377,14 +476,28 @@ ORLite - Extremely light weight SQLite-specific ORM
      'where first_name = ?',
      'Adam',
   );
+  
+  package Bar;
+  
+  # All available options enabled
+  # Some options shown are mutually exclusive, this would not actually run
+  
+  use ORLite {
+      package      => 'My::ORM',
+      file         => 'data/sqlite.db',
+      user_version => 12,
+      tables       => [ 'table1', 'table2' ],
+      readonly     => 1,
+      prune        => 1,
+      create       => sub {
+          my $dbh = shift;
+          $dbh->do('CREATE TABLE foo ( bar TEXT NOT NULL )');
+      },
+  );
 
 =head1 DESCRIPTION
 
-B<THIS CODE IS EXPERIMENTAL AND SUBJECT TO CHANGE WITHOUT NOTICE>
-
-B<YOU HAVE BEEN WARNED!>
-
-L<SQLite> is a light weight single file SQL database that provides an
+L<SQLite> is a light single file SQL database that provides an
 excellent platform for embedded storage of structured data.
 
 However, while it is superficially similar to a regular server-side SQL
@@ -403,12 +516,13 @@ L<DBIx::Class>.
 What this situation would seem to need is an object-relation system that is
 designed specifically for SQLite and is aligned with its idiosyncracies.
 
-ORLite is an object-relation system specifically for SQLite that follows
-many of the same principles as the ::Tiny series of modules and has a
-design that aligns directly to the capabilities of SQLite.
+ORLite is an object-relation system specifically tailored for SQLite that
+follows many of the same principles as the ::Tiny series of modules and
+has a design and feature set that aligns directly to the capabilities of
+SQLite.
 
 Further documentation will be available at a later time, but the synopsis
-gives a pretty good idea of how it will work.
+gives a pretty good idea of how it works.
 
 =head1 How it Works
 
@@ -635,9 +749,13 @@ For other issues, contact the author.
 
 Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
+=head1 SEE ALSO
+
+L<ORLite::Mirror>, L<ORLite::Migrate>
+
 =head1 COPYRIGHT
 
-Copyright 2008 Adam Kennedy.
+Copyright 2008 - 2009 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
